@@ -9,6 +9,7 @@ import pkg_resources
 from kocher_tools.multiplex import Multiplex
 from kocher_tools.logger import startLogger, logArgs
 from kocher_tools.deML import *
+from fastq_multx import fastqMultx
 
 def deMultiplexParser ():
 	'''
@@ -45,7 +46,12 @@ def deMultiplexParser ():
 	format_types = ['tsv', 'excel']
 	format_default = 'excel'
 	demultiplex_parser.add_argument('--index-format', help = 'Defines the format of the paired index', type = str, choices = format_types, default = format_default)
-	demultiplex_parser.add_argument('--excel-sheet', help = 'Defines the excel sheet to use. Default is the first sheet - i.e. 0', type = int, default = 0)
+	demultiplex_parser.add_argument('--excel-sheet', help = 'Defines the excel sheet to use. Default is the first sheet - i.e. 1', type = int, default = 1)
+
+	# Method options
+	method_types = ['fastq-multx', 'deML']
+	method_default = 'fastq-multx'
+	demultiplex_parser.add_argument('--method', help = 'Defines the demultiplex method. 1) fastq-multx - Fast, less reads; 2) deML Slow, more reads', type = str, choices = method_types, default = method_default)
 
 	# Map options
 	i5_revcomp_parser = demultiplex_parser.add_mutually_exclusive_group()
@@ -67,7 +73,7 @@ def deMultiplexParser ():
 	# Output arguments
 	demultiplex_parser.add_argument('--out-dir', help = 'Defines the output directory', type = str, default = 'Pipeline_Output')
 	demultiplex_parser.add_argument('--out-log', help = 'Defines the filename of the log file', type = str, default = 'demultiplex_pipeline.log')
-	demultiplex_parser.add_argument('--summary-log', help = 'Defines the filename of the log file', type = str, default = 'deML_summary.tsv')
+	demultiplex_parser.add_argument('--summary-log', help = 'Defines the filename of the log file', type = str, default = 'demultiplex_summary.tsv')
 	demultiplex_parser.add_argument('--overwrite', help = 'Defines if previous output should be overwritten', action = 'store_true')
 	
 	# Return the arguments
@@ -100,7 +106,11 @@ def main():
 	# Start the log
 	startLogger(demultiplex_args.out_log)
 	logArgs(demultiplex_args)
-	
+
+	# Update the excel sheet from 1-based to 0-based
+	if demultiplex_args.excel_sheet < 1: raise Exception(f'Unable to assign excel sheet: {demultiplex_args.excel_sheet}. Please note that --excel-sheet begin at 1 (i.e. 1-based indexing)')
+	excel_sheet_zero_based = demultiplex_args.excel_sheet - 1
+
 	# Check if all output should be kept, and if so, assign the arguments
 	if demultiplex_args.keep_all:
 		if demultiplex_args.keep_unknown or demultiplex_args.keep_failed or demultiplex_args.keep_indices:
@@ -109,23 +119,50 @@ def main():
 		# Assign arguments 
 		demultiplex_args.keep_unknown = demultiplex_args.keep_failed = demultiplex_args.keep_indices = True
 
-	# Create the demultiplex job using the index
-	demultiplex_job = deML.withIndex(demultiplex_args.paired_map, 
-									 demultiplex_args.index_format,
-									 pipeline_log_filename = demultiplex_args.out_log,
-									 deML_summary_filename = demultiplex_args.summary_log,
-									 excel_sheet = demultiplex_args.excel_sheet,
-									 keep_unknown = demultiplex_args.keep_unknown,
-									 keep_failed = demultiplex_args.keep_failed,
-									 keep_indices = demultiplex_args.keep_indices,
-									 i5_reverse_complement = demultiplex_args.i5_revcomp)
-	
-	# Demultiplex the following files
-	demultiplex_job.demultiplexFASTQs(out_dir = demultiplex_args.out_dir,
-									  i7_read_file = demultiplex_args.i7, 
-									  i5_read_file = demultiplex_args.i5, 
-									  r1_file = demultiplex_args.R1, 
-									  r2_file = demultiplex_args.R2)
+	# Demultiplex using fastq-multx
+	if demultiplex_args.method == 'fastq-multx':
+
+		# Create the demultiplex job using the index
+		demultiplex_job = fastqMultx.withIndex(demultiplex_args.paired_map, 
+										 demultiplex_args.index_format,
+										 pipeline_log_filename = demultiplex_args.out_log,
+										 fastq_multx_summary_filename = demultiplex_args.summary_log,
+										 excel_sheet = excel_sheet_zero_based,
+										 keep_unknown = demultiplex_args.keep_unknown,
+										 keep_indices = demultiplex_args.keep_indices,
+										 i5_reverse_complement = demultiplex_args.i5_revcomp)
+		
+		# Demultiplex the following files
+		demultiplex_job.demultiplexFASTQs(out_dir = demultiplex_args.out_dir,
+										  i7_read_file = demultiplex_args.i7, 
+										  i5_read_file = demultiplex_args.i5, 
+										  r1_file = demultiplex_args.R1, 
+										  r2_file = demultiplex_args.R2)
+
+	# Demultiplex using deML
+	elif demultiplex_args.method == 'deML': 
+
+		# Create the demultiplex job using the index
+		demultiplex_job = deML.withIndex(demultiplex_args.paired_map, 
+										 demultiplex_args.index_format,
+										 pipeline_log_filename = demultiplex_args.out_log,
+										 deML_summary_filename = demultiplex_args.summary_log,
+										 excel_sheet = excel_sheet_zero_based,
+										 keep_unknown = demultiplex_args.keep_unknown,
+										 keep_failed = demultiplex_args.keep_failed,
+										 keep_indices = demultiplex_args.keep_indices,
+										 i5_reverse_complement = demultiplex_args.i5_revcomp)
+		
+		# Demultiplex the following files
+		demultiplex_job.demultiplexFASTQs(out_dir = demultiplex_args.out_dir,
+										  i7_read_file = demultiplex_args.i7, 
+										  i5_read_file = demultiplex_args.i5, 
+										  r1_file = demultiplex_args.R1, 
+										  r2_file = demultiplex_args.R2)
+
+	else:
+		raise Exception(f'Unrecognized method: {demultiplex_args.method}')
+
 
 if __name__== "__main__":
 	main()
