@@ -1,5 +1,6 @@
 import os
 import sys
+import math
 import random
 import string
 import logging
@@ -47,8 +48,19 @@ class Plink2 (list):
 			raise IOError(f'Cannot find VCF ({self.vcf_filename}) or Binary-PED ({self.bed_prefix}.*). PLINK2 requires either a VCF or Binary-PED file to operate')
 
 		# Create the Binary-PED, if needed
-		if self.vcf_exists and not self.bed_exists: self._cvtToPlink()
+		if not self.bed_exists:
+			
+			# Assign the file prefix for the vcf filename
+			if self.vcf_filename.endswith('.vcf.gz'): file_prefix = self.vcf_filename[:-7]
+			elif self.vcf_filename.endswith('.vcf'): file_prefix = self.vcf_filename[:-4]
+			elif self.vcf_filename.endswith('.bcf'): file_prefix = self.vcf_filename[:-4]
+			
+			# Create the Binary-PED
+			if not os.path.isfile(f'{file_prefix}.bed') or not os.path.isfile(f'{file_prefix}.bim') or not os.path.isfile(f'{file_prefix}.fam'): self._cvtToPlink()
 
+			# Update the prefix
+			self.bed_prefix = file_prefix
+			
 		# Assign the input/output arg
 		if self.bed_prefix: self._plink2_call_args.extend(['--bfile', self.bed_prefix])
 		else: raise IOError('No Binary-PED prefix assigned')
@@ -67,7 +79,7 @@ class Plink2 (list):
 		else: raise Exception(f'VCF has non-standard file extension. Please use .vcf or .vcf.gz')
 
 		# Assign the conversion args
-		self._plink2_call_args.extend(['--vcf', self.vcf_filename, '--make-bed', '--out', self.bed_prefix, '--double-id', '--allow-extra-chr'])
+		self._plink2_call_args.extend(['--vcf', self.vcf_filename, '--make-bed', '--out', self.bed_prefix, '--double-id', '--allow-extra-chr', '--set-missing-var-ids', '@:#:$r,$a'])
 
 		# Call plink2
 		self._call()
@@ -122,13 +134,14 @@ class Plink2 (list):
 			if include_samples or exclude_samples: raise Exception('--include-samples/--exclude-samples cannot be used with a model')
 			self._plink2_call_args.extend(['--keep', self._sampleFile(list(self.sample_model_dict))])
 
-	def calcPCA (self, pca_type = 'sample-wts', pc_count = 10, pca_modifier = None):
+	def calcPCA (self, pca_type = 'sample-wts', pc_count = 10, pca_modifier = None, afreq_filename = ''):
 
 		# Set the PCA arguments
 		self._plink2_call_args.append('--pca')
 		if pca_type != 'sample-wts': self._plink2_call_args.append(pca_type)
 		self._plink2_call_args.append('10')
 		if pca_modifier: self._plink2_call_args.append(pca_modifier)
+		if afreq_filename: self._plink2_call_args.extend(['--read-freq', afreq_filename])
 
 		# Call plink2
 		self._call()
@@ -136,6 +149,18 @@ class Plink2 (list):
 		# Cleanup and rename the log
 		os.rename(f'{self.out_prefix}.log', f'{self.out_prefix}.pca.log')
 		self._cleanup()
+
+	def calcAlleleFreqs (self):
+
+		# Set the Allele Frequency arguments
+		self._plink2_call_args.append('--freq')
+
+		# Call plink2
+		self._call()
+
+		# Cleanup and rename the log
+		os.rename(f'{self.out_prefix}.log', f'{self.out_prefix}.afreq.log')
+		self._cleanup()
 		
 	def calcFst (self, method = 'hudson', category = 'Pops', report_variants = True):
 
@@ -172,8 +197,53 @@ class Plink2 (list):
 		os.rename(f'{self.out_prefix}.log', f'{self.out_prefix}.fst.log')
 		self._cleanup()
 
+	def binFst (self, window_size, window_step, remove_site_fst = False):
+
+		# Loop site-based Fst files
+		for site_fst_filename in os.listdir(self.out_dir):
+			if not site_fst_filename.endswith('.fst.var'): continue
+			site_fst_path = os.path.join(self.out_dir, site_fst_filename)
+
+			# If specified, remove the site Fst files
+			if remove_site_fst: self._files_to_remove.append(site_fst_path)
+
+			# Create a list to store the binned Fst data
+			binned_fst_data = []
+
+			# Open the site-based Fst file
+			site_fst_dataframe = pd.read_csv(site_fst_path, sep = '\t')
+
+			# Assign the Fst column
+			if site_fst_dataframe.columns[4] in ['WC_FST', 'HUDSON_FST']: fst_column = site_fst_dataframe.columns[4]
+			else: raise Exception (f'Unable to assign Fst column from: {site_fst_dataframe.columns}')
+
+			# Split by chromosome
+			for chrom, chrom_site_fst_dataframe in site_fst_dataframe.groupby(by=['#CHROM']):
+
+				# Assign maximum length 
+				chrom_max = chrom_site_fst_dataframe['POS'].max()
+				max_len = math.ceil(chrom_max/window_size) * window_size
+
+				# Bin Fst data
+				for bin_start in range(1, max_len, window_step):
+					bin_end = bin_start + window_size - 1
+					binned_site_fst_dataframe = chrom_site_fst_dataframe[chrom_site_fst_dataframe['POS'].between(bin_start, bin_end)]
+					binned_site_fst_dataframe = binned_site_fst_dataframe.dropna(subset=[fst_column])
+					binned_fst_data.append({'#CHROM':chrom, 
+											'BIN_START':bin_start, 
+											'BIN_END':bin_end, 
+											'N_VARIANTS':binned_site_fst_dataframe.shape[0], 
+											'MEAN_FST':binned_site_fst_dataframe[fst_column].mean()})
+
+			# Save the binned Fst into a file
+			pd.DataFrame(binned_fst_data).to_csv(site_fst_path[:-7] + 'windowed.fst.var', sep = '\t', index = False, na_rep = 'nan')
+
+		# Cleanup, if needed
+		self._cleanup()
+
 	def _cleanup (self):
 		for file_to_remove in self._files_to_remove: os.remove(file_to_remove)
+		self._files_to_remove = []
 
 	def _sampleFile (self, samples, prefix = 'Samples', str_len = 12):
 		rand_str = ''.join(random.choice(string.ascii_uppercase + string.digits) for i in range(str_len))
